# THIS FILE IS GENERATED FROM macrosynergy SETUP.PY
short_version: str = '0.0.2'
version: str = '0.0.2'
full_version: str = '0.0.2'
git_revision: str = 'edd9cf47cb561ddda5fadaeaed107c766f88b1f2'
release: bool = True
if not release:
    version = full_version



import unittest
import numpy as np
import pandas as pd

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.check_availability import check_startyears, check_enddates

cids = ['AUD', 'CAD', 'GBP']
xcats = ['CRY', 'XR']
df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
df_cids.loc['AUD',] = ['2010-01-01', '2020-12-31', 0.5, 2]
df_cids.loc['CAD',] = ['2011-01-01', '2020-11-30', 0, 1]
df_cids.loc['GBP',] = ['2012-01-01', '2020-11-30', -0.2, 0.5]

df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
df_xcats.loc['CRY',] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]
df_xcats.loc['XR',] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]

dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

df_sy = check_startyears(dfd)
df_ed = check_enddates(dfd)


class Test_All(unittest.TestCase):

    def test_check_startyears(self):

        df_sy = check_startyears(dfd)
        df_exp = pd.DataFrame(data=np.zeros((3, 2)), index=cids, columns=xcats)
        for cid in cids:
            for xcat in xcats:
                cid_max = pd.Series(pd.to_datetime(df_cids.loc[cid, 'earliest'])).dt.year
                xcat_max = pd.Series(pd.to_datetime(df_xcats.loc[xcat, 'earliest'])).dt.year
                df_exp.loc[cid, xcat] = np.max([cid_max, xcat_max])
        self.assertTrue((df_sy.astype(int)).equals(df_exp.astype(int)))

    def test_check_enddates(self):

        df_ed = check_enddates(dfd)
        df_exp = pd.DataFrame(data=np.zeros((3, 2)), index=cids, columns=xcats)
        for cid in cids:
            for xcat in xcats:
                cid_min = pd.to_datetime(df_cids.loc[cid, 'latest'])
                xcat_min = pd.to_datetime(df_xcats.loc[xcat, 'latest'])
                df_exp.loc[cid, xcat] = np.min([cid_min, xcat_min]).strftime("%Y-%m-%d")
        self.assertTrue(df_ed.equals(df_exp))


if __name__ == '__main__':

    unittest.main()



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.check_availability import reduce_df


def view_timelines(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None,
                   start: str = '2000-01-01', end: str = None, val: str = 'value', cumsum: bool = False,
                   title: str = None, title_adj=0.95,
                   ncol: int = 3, same_y: bool = True, size: Tuple[float] = (12, 7), aspect: float = 1.7):

    """Display facet grid of time lines of one or more categories

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date' and at least one column with values of interest.
    :param <List[str]> xcats: extended categories to be checked on. Default is all in the dataframe.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.
        If this contains only one cross section a single chart is created rather than a facet grid.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <str> val: name of column that contains the values of interest. Default is 'value'.
    :param <bool> cumsum: chart the cumulative sum of the value. Default is False.
    :param <str> title: chart heading. Default is no title.
    :param <float> title_adj: parameter that positions title relative to the facet. Default is 0.95.
    :param <int> ncol: number of columns in facet. Default is 3.
    :param <bool> same_y: if True (default) all axis plots in the facet share the same y axis.
    :param <Tuple[float]> size: two-element tuple setting width/height of figure. Default is (12, 7).
    :param <float> aspect: width-height ratio for plots in facet.

    """

    df, xcats, cids = reduce_df(df, xcats, cids, start, end, out_all=True)

    if cumsum:
        df[val] = df.sort_values(['cid', 'xcat', "real_date"])[['cid', 'xcat', val]].groupby(['cid', 'xcat']).cumsum()

    sns.set(rc={'figure.figsize': size}, style='darkgrid')
    if len(cids) == 1:
        ax = sns.lineplot(data=df, x='real_date', y=val, hue='xcat', ci=None)
        plt.axhline(y=0, c=".5")
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(handles=handles[0:], labels=labels[0:])
        ax.set_xlabel("")
        ax.set_ylabel("")
        if title is not None:
            plt.title(title)
    else:
        fg = sns.FacetGrid(data=df, col='cid', col_wrap=ncol, sharey=same_y, aspect=aspect, col_order=cids)
        fg.map_dataframe(sns.lineplot, x='real_date', y=val, hue='xcat', ci=None)
        fg.map(plt.axhline, y=0, c=".5")
        fg.set_titles(col_template='{col_name}')
        fg.add_legend()
        if title is not None:
            fg.fig.subplots_adjust(top=title_adj)
            fg.fig.suptitle(title, fontsize=20)
    plt.show()


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP', 'NZD']
    xcats = ['XR', 'CRY']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.2, 0.2]
    df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP', ] = ['2012-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD', ] = ['2012-01-01', '2020-09-30', -0.1, 3]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.95, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    view_timelines(dfd, xcats=['XR', 'CRY'], cids=cids[0], ncol=1, title='AUD return and carry')
    view_timelines(dfd, xcats=['CRY'], cids=cids, ncol=2, title='Carry')
    view_timelines(dfd, xcats=['CRY', 'XR'], cids=cids, ncol=2, title='Carry and return')
    view_timelines(dfd, xcats=['XR'], cids=cids, ncol=2, cumsum=True, same_y=False, aspect=2)


    dfd.info()


import unittest
import random
import numpy as np
import pandas as pd

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.shape_dfs import reduce_df, categories_df


cids = ['AUD', 'CAD', 'GBP']
xcats = ['CRY', 'XR']
df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
df_cids.loc['AUD',] = ['2010-01-01', '2020-12-31', 0.5, 2]
df_cids.loc['CAD',] = ['2011-01-01', '2020-11-30', 0, 1]
df_cids.loc['GBP',] = ['2012-01-01', '2020-11-30', -0.2, 0.5]

df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
df_xcats.loc['CRY',] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]
df_xcats.loc['XR',] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]

random.seed(1)
dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)


class Test_All(unittest.TestCase):

    def test_reduce_df(self):

        dfd_x = reduce_df(dfd, xcats=['CRY'], cids=cids[0:2], start='2013-01-01', end='2019-01-01')
        self.assertTrue(all(dfd_x['cid'].unique() == np.array(['AUD', 'CAD'])))
        self.assertTrue(all(dfd_x['xcat'].unique() == np.array(['CRY'])))
        self.assertTrue(dfd_x['real_date'].min() == pd.to_datetime('2013-01-01'))
        self.assertTrue(dfd_x['real_date'].max() == pd.to_datetime('2019-01-01'))

    def test_reduce_df_black(self):

        black = {'AUD_1': ['2011-01-01', '2012-12-31'], 'AUD_2': ['2018-01-01', '2100-01-01']}
        dfd_x = reduce_df(dfd, xcats=['XR'], cids=cids, blacklist=black)
        dfd_aud = dfd_x[dfd_x['cid'] == 'AUD']
        self.assertTrue(dfd_aud['real_date'].min() == pd.to_datetime('2010-01-01'))
        black_range_1 = pd.date_range(start='2011-01-01', end='2012-12-31')
        self.assertTrue(not any(item in black_range_1 for item in dfd_aud['real_date']))

        black = {'CAD': ['2014-01-01', '2014-12-31']}
        dfd_x = reduce_df(dfd, xcats=['XR'], cids=cids, blacklist=black)
        dfd_cad = dfd_x[dfd_x['cid'] == 'CAD']
        black_range_2 = pd.date_range(start='2014-01-01', end='2014-12-31')
        self.assertTrue(not any(item in black_range_2 for item in dfd_cad['real_date']))

    def test_categories_df_conversion(self):

        dfc = categories_df(dfd, xcats=['XR', 'CRY'], cids=cids, freq='M', lag=0, xcat_aggs=['mean', 'last'])

        filt1 = (dfd['real_date'].dt.year == 2011) & (dfd['real_date'].dt.month == 10)
        filt2 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'XR')
        x1 = dfd[filt1 & filt2]['value'].mean()
        x2 = float(dfc.loc[('AUD', '2011-10-31'), 'XR'])
        self.assertTrue(x1 == x2)  # check correct application of monthly mean

        filt2 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'CRY')
        x1 = dfd[filt1 & filt2]['value'].iloc[-1]
        x2 = float(dfc.loc[('AUD', '2011-10-31'), 'CRY'])
        self.assertTrue(x1 == x2)  # check correct application of end-of-month aggregation

    def test_categories_df_lags(self):

        dfc = categories_df(dfd, xcats=['XR', 'CRY'], cids=cids, freq='M', lag=1, fwin=3, xcat_aggs=['mean', 'last'])

        filt1 = (dfd['real_date'].dt.year == 2013) & (dfd['real_date'].dt.month.isin([10, 11, 12]))
        filt2 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'XR')
        x1 = round(float(np.mean(dfd[filt1 & filt2].set_index('real_date').resample('M').mean())), 10)
        x2 = round(float(dfc.loc[('AUD', '2013-10-31'), 'XR']), 10)
        self.assertTrue(x1 == x2)  # check correct application of 1st series forward (average) window

        filt1 = (dfd['real_date'].dt.year == 2011) & (dfd['real_date'].dt.month == 10)
        filt2 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'CRY')
        x1 = dfd[filt1 & filt2]['value'].iloc[-1]
        x2 = float(dfc.loc[('AUD', '2011-11-30'), 'CRY'])
        self.assertTrue(x1 == x2)  # check correct application of 2nd series lag

    def test_categories_df_black(self):

        black = {'CAD': ['2014-01-01', '2014-12-31']}
        dfc = categories_df(dfd, xcats=['XR', 'CRY'], cids=cids, freq='M', xcat_aggs=['mean', 'last'], blacklist=black)
        dfc_cad = dfc[np.array(dfc.reset_index(level=0)['cid']) == 'CAD']
        black_range_1 = pd.date_range(start='2014-01-01', end='2014-12-31')
        self.assertTrue(len([item for item in dfc_cad.reset_index()['real_date'] if item in black_range_1]) == 0)


if __name__ == '__main__':

    unittest.main()


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.shape_dfs import reduce_df


def missing_in_df(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None):
    """Print cross sections and extended categories that are missing or redundant in the dataframe

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date'.
    :param <List[str]> xcats: extended categories to be checked on. Default is all in the dataframe.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.

    """
    print('Missing xcats across df: ', set(xcats) - set(df['xcat'].unique()))  # any xcats missing
    xcats_used = sorted(list(set(xcats).intersection(set(df['xcat'].unique()))))
    for xcat in xcats_used:
        cids_xcat = df.loc[df['xcat'] == xcat, 'cid'].unique()
        print(f'Missing cids for {xcat}: ', set(cids) - set(cids_xcat))  # any cross section missing?


def check_startyears(df: pd.DataFrame):
    """
    Dataframe with starting years across all extended categories and cross sections

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
    'cid', 'xcats', 'real_date'

    """
    df_starts = df[['cid', 'xcat', 'real_date']].groupby(['cid', 'xcat']).min()
    df_starts['real_date'] = pd.DatetimeIndex(df_starts.loc[:, 'real_date']).year

    return df_starts.unstack().loc[:, 'real_date'].astype(int, errors='ignore')


def check_enddates(df: pd.DataFrame):
    """
    Dataframe with end dates across all extended categories and cross sections

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
    'cid', 'xcats', 'real_date'
    """

    df_ends = df[['cid', 'xcat', 'real_date']].groupby(['cid', 'xcat']).max()
    df_ends['real_date'] = df_ends['real_date'].dt.strftime('%Y-%m-%d')

    return df_ends.unstack().loc[:, 'real_date']


def visual_paneldates(df: pd.DataFrame, size: Tuple[float] = None):
    """
    Visualize panel dates with color codes.

    :param <pd.Dataframe> df: dataframe cross sections rows and category columns
    :param <Tuple[float]> size: tuple of floats with width/length of displayed heatmap

    """

    if all(df.dtypes == object):

        df = df.apply(pd.to_datetime)
        maxdate = df.max().max()
        df = (maxdate - df).apply(lambda x: x.dt.days)
        header = f"Missing days prior to {maxdate.strftime('%Y-%m-%d')}"

    else:

        header = 'Start years of quantamental indicators'

    if size is None:
        size = (max(df.shape[0] / 2, 15), max(1, df.shape[1]/ 2))
    sns.set(rc={'figure.figsize': size})
    sns.heatmap(df.T, cmap='Reds', center=df.stack().mean(), annot=True, fmt='.0f', linewidth=1, cbar=False)
    plt.xlabel('')
    plt.ylabel('')
    plt.title(header, fontsize=18)
    plt.show()


def check_availability(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None, start: str = None,
                       start_size: Tuple[float] = None, end_size: Tuple[float] = None):
    """
    Wrapper for visualizing start and end dates of a filtered dataframe.

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date'.
    :param <List[str]> xcats: extended categories to be checked on. Default is all in the dataframe.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.
    :param <str> start: string representing earliest considered date. Default is None.
    :param <Tuple[float]> start_size: tuple of floats with width/length of start years heatmap. Default is None.
    :param <Tuple[float]> end_size: tuple of floats with width/length of end dates heatmap. Default is None.

    """
    dfx = reduce_df(df, xcats=xcats, cids=cids, start=start)
    dfs = check_startyears(dfx)
    visual_paneldates(dfs, size=start_size)
    dfe = check_enddates(dfx)
    visual_paneldates(dfe, size=end_size)


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP']
    xcats = ['XR', 'CRY']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.5, 2]
    df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP', ] = ['2012-01-01', '2020-11-30', -0.2, 0.5]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]
    df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    xxcats = xcats+['TREND']
    xxcids = cids+['USD']
    missing_in_df(dfd, xcats=xxcats, cids=xxcids)

    df_sy = check_startyears(dfd)
    print(df_sy)
    visual_paneldates(df_sy)
    df_ed = check_enddates(dfd)
    print(df_ed)
    visual_paneldates(df_ed)
    df_ed.info()


from .view_ranges import view_ranges
from .view_timelines import view_timelines
from .view_correlations import correl_matrix
from .category_relations import CategoryRelations
from .view_grades import view_grades

__all__ = ['view_ranges', 'view_timelines', "correl_matrix", 'CategoryRelations', 'view_grades']


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn import metrics as skm
from scipy import stats 

from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.shape_dfs import categories_df


class SignalReturnRelations:

    """Class for analyzing and visualizing relations between a signal and subsequent return

    :param <pd.Dataframe> df: standardized data frame with the following necessary columns:
        'cid', 'xcats', 'real_date' and 'value.
    :param <str> ret: return category.
    :param <str> sig: signal category.
    :param <List[str]> cids: cross sections to be considered. Default is all in the data frame.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <dict> blacklist: cross sections with date ranges that should be excluded from the data frame.
    :param <str> freq: letter denoting frequency at which the series are to be sampled.
        This must be one of 'D', 'W', 'M', 'Q', 'A'. Default is 'M'.
    :param <int> fwin: Forward window of return category in base periods. Default is 1.

    """
    def __init__(self, df: pd.DataFrame, ret: str, sig: str, cids: List[str] = None,
                 start: str = None, end: str = None, fwin: int = 1, blacklist: dict = None,
                 freq: str = 'M'):

        self.df = categories_df(df, [ret, sig], cids, 'value', start=start, end=end, freq=freq, blacklist=blacklist,
                                lag=1, fwin=fwin, xcat_aggs=['mean', 'last'])
        self.ret = ret
        self.sig = sig
        self.freq = freq
        self.cids = list(np.sort(self.df.index.get_level_values(0).unique()))
        self.df_cs = self.panel_relations(cs_type='cids')
        self.df_ys = self.panel_relations(cs_type='years')
        self.dic_freq = {'D': 'daily', 'W': 'weekly', 'M': 'monthly', 'Q': 'quarterly', 'A': 'annual'}

        """Creates a dataframe of return and signal in the appropriate form for subsequent analysis."""

    def panel_relations(self, cs_type: str = 'cids'):
        """Creates a dataframe with information on the signal-return relation across cids/years and the panel."""

        assert cs_type in ['cids', 'years']
        if cs_type == 'cids':
            df = self.df.dropna(how='any')
            css = self.cids
        else:
            df = self.df.dropna(how='any')
            df['year'] = np.array(df.reset_index(level=1)['real_date'].dt.year)
            css = [str(i) for i in df['year'].unique()]

        statms = ['accuracy', 'bal_accuracy', 'f1_score', 'pearson', 'pearson_pval', 'kendall', 'kendall_pval']
        df_out = pd.DataFrame(index=['Panel', 'Mean', 'PosRatio'] + css, columns=statms)

        for cs in (css + ['Panel']):
            if cs in css:
                if cs_type == 'cids':
                    df_cs = df.loc[cs,]
                else:
                    df_cs = df[df['year'] == float(cs)]
            elif cs == 'Panel':
                df_cs = df

            ret_signs, sig_signs = np.sign(df_cs[self.ret]), np.sign(df_cs[self.sig])
            df_out.loc[cs, 'accuracy'] = skm.accuracy_score(sig_signs, ret_signs)
            df_out.loc[cs, 'bal_accuracy'] = skm.balanced_accuracy_score(sig_signs, ret_signs)
            df_out.loc[cs, 'f1_score'] = skm.f1_score(sig_signs, ret_signs, average='weighted')

            ret_vals, sig_vals = df_cs[self.ret], df_cs[self.sig]
            df_out.loc[cs, ['kendall', 'kendall_pval']] = stats.kendalltau(ret_vals, sig_vals)
            df_out.loc[cs, ['pearson', 'pearson_pval']] = stats.pearsonr(ret_vals, sig_vals)

        df_out.loc['Mean', :] = df_out.loc[css, :].mean()

        above50s = statms[0:3]
        df_out.loc['PosRatio', above50s] = (df_out.loc[css, above50s] > 0.5).mean()
        above0s = [statms[i] for i in [3, 5]]
        df_out.loc['PosRatio', above0s] = (df_out.loc[css, above0s] > 0).mean()
        below50s = [statms[i] for i in [4, 6]]
        pos_pvals = np.mean(np.array(df_out.loc[css, below50s] < 0.5) * np.array(df_out.loc[css, above0s] > 0), axis=0)
        df_out.loc['PosRatio', below50s] = pos_pvals  # positive correlations with error probabilities < 50%
        return df_out

    def cross_section_table(self):
        """Returns a dataframe with information on the signal-return relation across sections and the panel."""
        return self.df_cs.round(decimals=3)

    def yearly_table(self):
        """Returns dataframe with information on the signal-return relation across years and the panel."""
        return self.df_ys.round(decimals=3)

    def accuracy_bars(self, type: str = 'cross_section', title: str = None, size: Tuple[float] = None,
                      legend_pos: str = 'best'):
        """Bars of overall and balanced accuracy

        :param <str> type: type of segment over which bars are drawn. Must be 'cross_section' (default) or 'years'
        :param <str> title: chart header. Default will be applied if none is chosen.
        :param <Tuple[float]> size: 2-tuple of width and height of plot.  Default will be applied if none is chosen.
        :param <str> legend_pos: position of legend box. Default is 'best'. See matplotlib.pyplot.legend.

        """

        assert type in ['cross_section', 'years']

        df_xs = self.df_cs if type == 'cross_section' else self.df_ys
        dfx = df_xs[~df_xs.index.isin(['PosRatio'])]

        if title is None:
            title = f'Accuracy for sign prediction of {self.ret} based on {self.sig} ' \
                    f'at {self.dic_freq[self.freq]} frequency'
        if size is None:
            size = (np.max([dfx.shape[0]/2, 8]), 6)

        plt.style.use('seaborn')
        plt.figure(figsize=size)
        x_indexes = np.arange(len(dfx.index))  # generic x index
        w = 0.4  # offset parameter, related to width of bar
        plt.bar(x_indexes - w / 2, dfx['accuracy'], label='Accuracy', width=w, color='lightblue')
        plt.bar(x_indexes + w / 2, dfx['bal_accuracy'], label='Balanced Accuracy', width=w, color='steelblue')
        plt.xticks(ticks=x_indexes, labels=dfx.index, rotation=0)  # customize x ticks/labels
        plt.axhline(y=0.5, color='black', linestyle='-', linewidth=0.5)
        plt.ylim(np.round(np.max(dfx.loc[:, ['accuracy', 'bal_accuracy']].min().min()-0.03, 0), 2))
        plt.title(title)
        plt.legend(loc=legend_pos)
        plt.show()

    def correlation_bars(self, type: str = 'cross_section', title: str = None, size: Tuple[float] = None,
                         legend_pos: str = 'best'):
        """Correlation coefficients and significance

        :param <str> type: type of segment over which bars are drawn. Must be 'cross_section' (default) or 'years'
        :param <str> title: chart header. Default will be applied if none is chosen.
        :param <Tuple[float]> size: 2-tuple of width and height of plot.  Default will be applied if none is chosen.
        :param <str> legend_pos: position of legend box. Default is 'best'. See matplotlib.pyplot.legend.

        """

        df_xs = self.df_cs if type == 'cross_section' else self.df_ys
        dfx = df_xs[~df_xs.index.isin(['PosRatio', 'Mean'])]

        pprobs = np.array([(1 - pv) * (np.sign(cc) + 1) / 2 for pv, cc in zip(dfx['pearson_pval'], dfx['pearson'])])
        pprobs[pprobs == 0] = 0.01  # token small value for bar
        kprobs = np.array([(1 - pv) * (np.sign(cc) + 1) / 2 for pv, cc in zip(dfx['kendall_pval'], dfx['kendall'])])
        kprobs[kprobs == 0] = 0.01  # token small value for bar

        if title is None:
            title = f'Positive correlation probability of {self.ret} and lagged {self.sig} ' \
                    f'at {self.dic_freq[self.freq]} frequency'
        if size is None:
            size = (np.max([dfx.shape[0]/2, 8]), 6)

        plt.style.use('seaborn')
        plt.figure(figsize=size)
        x_indexes = np.arange(len(dfx.index))  # generic x index
        w = 0.4  # offset parameter, related to width of bar
        plt.bar(x_indexes - w / 2, pprobs, label='Pearson', width=w, color='lightblue')
        plt.bar(x_indexes + w / 2, kprobs, label='Kendall', width=w, color='steelblue')
        plt.xticks(ticks=x_indexes, labels=dfx.index, rotation=0)  # customize x ticks/labels
        plt.axhline(y=0.95, color='orange', linestyle='--', linewidth=0.5, label='95% probability')
        plt.axhline(y=0.99, color='red', linestyle='--', linewidth=0.5, label='99% probability')
        plt.title(title)
        plt.legend(loc=legend_pos)
        plt.show()


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP', 'NZD']
    xcats = ['XR', 'CRY', 'GROWTH', 'INFL']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD',] = ['2000-01-01', '2020-12-31', 0.1, 1]
    df_cids.loc['CAD',] = ['2001-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP',] = ['2002-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD',] = ['2002-01-01', '2020-09-30', -0.1, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR',] = ['2000-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY',] = ['2000-01-01', '2020-10-30', 1, 2, 0.95, 1]
    df_xcats.loc['GROWTH',] = ['2001-01-01', '2020-10-30', 1, 2, 0.9, 1]
    df_xcats.loc['INFL',] = ['2001-01-01', '2020-10-30', 1, 2, 0.8, 0.5]

    black = {'AUD': ['2006-01-01', '2015-12-31'], 'GBP': ['2012-01-01', '2100-01-01']}

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    srr = SignalReturnRelations(dfd, sig='CRY', ret='XR', freq='D', blacklist=black)
    srr.correlation_bars(type='cross_section')
    srr.accuracy_bars(type='cross_section')
    df_cs_stats = srr.cross_section_table()
    df_ys_stats = srr.yearly_table()
    print(df_cs_stats)
    print(df_ys_stats)


import numpy as np
import pandas as pd
from typing import List, Union, Tuple
import matplotlib.pyplot as plt
import math
import datetime as dt


class VintageData:
    """Creates standardized dataframe of single-ticker vintages

    This class creates standardized grade 1 and grade 2 vintage data

    :param <str> ticker: ticker name
    :param <str> cutoff: last possible release date. The format must be '%Y-%m-%d'.
            All other dates are calculated from this one. Default is end 2020.
    :param <list> release_lags: list of integers in ascending order denoting lags of the first, second etc. release
            in (calendar) days. Default is first release after 15 days and revision after 30 days.
            If days fall on weekend they will be delayed to  Monday.
    :param <int> number_firsts: number of first-release vintages in the simulated data set. Default is 24.
    :param <int> shortest: number of observations in the first (shortest) vintage. Default is 36.
    :param <str> freq: letter denoting the frequency of the vintage data. Must be one of 'M' (monthly, default),
            'Q' (quarterly)  or 'W' (weekly).
    :param <float> start_value: expected first value of the random series. Default is 100.
    :param <float> trend_ar: annualized trend. Default is 5% linear drift per year.
            This is applied to the start value. If the start value is not positive the linear trend is added as number.
    :param <float> sd_ar: annualized standard deviation. Default is sqrt(12).
    :param <float> seasonal: adds seasonal pattern (applying linear factor from low to high through the year)
            with value denoting the average % seasonal factor through the year. Default is None.
            The seasonal pattern makes only sense for values that are strictly positive and are interpreted as indices.
    :param <int> added_dates: number of added first release dates, used for grade 2 dataframe generation.
            Default is 12.

    """

    def __init__(self, ticker, cutoff='2020-12-31', release_lags=[15, 30], number_firsts=24, shortest=36, freq='M',
                 start_value=100, trend_ar=5, sd_ar=math.sqrt(12), seasonal=None, added_dates=12):
        self.ticker = ticker
        self.cutoff = dt.datetime.strptime(cutoff, '%Y-%m-%d').date()
        self.release_lags = release_lags
        self.number_firsts = number_firsts
        self.shortest = shortest

        self.freq = freq
        self.af = int(pd.DataFrame([4, 12, 52], index=['Q', 'M', 'W']).loc[freq])

        self.start_value = start_value
        self.trend_ar = trend_ar
        self.sd_ar = sd_ar
        self.seasonal = seasonal

        self.number_firsts_gr2 = self.number_firsts + added_dates

    def make_grade1(self):
        ref_date = (self.cutoff - dt.timedelta(self.release_lags[0])).replace(day=1)
        eop_dates = pd.date_range(end=ref_date, periods=self.number_firsts, freq=self.freq).date
        vin_lengths = [vl for vl in range(self.shortest, self.shortest + self.number_firsts)]
        df_gr1 = pd.DataFrame(columns=['release_date', 'observation_date', 'value'])
        for eop_date in eop_dates:
            for rl in self.release_lags:
                rel_date = eop_date + dt.timedelta(days=rl)
                if rel_date.weekday() >= 5:  # shift release dates out of weekends
                    rel_date = rel_date - dt.timedelta(days=1) if (rel_date.weekday() == 5) \
                        else rel_date + dt.timedelta(days=1)
                vin_length = vin_lengths[int(np.where(eop_dates == eop_date)[0])]  # pick vintage length
                obs_dates = pd.date_range(end=eop_date, periods=vin_length, freq=self.freq)
                if self.start_value > 0:
                    trend = self.start_value * (1+np.array([i * self.trend_ar/self.af for i in range(vin_length)])/100)
                else:  # if trend cannot be proportionate just add as number
                    trend = self.start_value + np.array([i * self.trend_ar/self.af for i in range(vin_length)])
                values = trend + np.random.normal(0, self.sd_ar / math.sqrt(self.af), vin_length)
                if self.seasonal is not None:
                    linear_scale = list(range(self.af)) - np.mean(list(range(self.af)))
                    seas_factors = (self.seasonal / np.std(linear_scale)) * linear_scale
                    if self.freq == 'M':
                        values = values * (1 + seas_factors[obs_dates.month - 1]/100)
                    if self.freq == 'Q':
                        values = values * (1 + seas_factors[(obs_dates.month/3 - 1).astype(np.int64)]/100)
                    if self.freq == 'W':
                        values = values * (1 + seas_factors[np.clip(pd.Int64Index(obs_dates.isocalendar().week) - 1,
                                                                    a_min=0, a_max=51)] / 100)
                df_rel = pd.DataFrame({'release_date': rel_date, 'observation_date': obs_dates.date, 'value': values})
                df_gr1 = df_gr1.append(df_rel)

        df_gr1["grading"] = "1"

        return self.add_ticker_parts(df_gr1)

    def make_graded(self, grading, upgrades=[]):

        """
        Simulates an explicitly graded dataframe with a column 'grading'
        :param <list> grading: optional addition of grading column. List of grades used from lowest to highest.
        Default is None. Must be a subset of [3, 2.2, 2.1, 1].
        :param <list> upgrades: indices of release dates at which the series upgrade.
        Must have length of grading minus one. Default is None.
        """
        assert len(upgrades) == (len(grading) - 1)
        df_grd = self.make_grade1()
        df_grd['grading'] = str(grading[0])
        for up in range(len(upgrades)):
            filt = df_grd['release_date'] >= df_grd['release_date'].unique()[upgrades[up]]
            df_grd.loc[filt, 'grading'] = str(grading[up + 1])
        return df_grd

    def make_grade2(self):
        ref_date = (self.cutoff - dt.timedelta(self.release_lags[0])).replace(day=1)
        eop_dates = pd.date_range(end=ref_date, periods=self.number_firsts_gr2, freq=self.freq).date
        df_gr2 = pd.DataFrame(columns=['release_date', 'observation_date'])
        for eop_date in eop_dates:
            for rl in self.release_lags:
                rel_date = eop_date + dt.timedelta(days=rl)
                if rel_date.weekday() >= 5:  # shift release dates out of weekends
                    rel_date = rel_date - dt.timedelta(days=1) if (rel_date.weekday() == 5) \
                        else rel_date + dt.timedelta(days=1)
                df_gr2.loc[len(df_gr2)] = [rel_date, eop_date]
        df_gr2['ticker'] = self.ticker
        return self.add_ticker_parts(df_gr2)

    def add_ticker_parts(self, df):
        old_cols = list(df.columns)
        add_cols = ['cross_section', 'category_code', 'adjustment', 'transformation']
        ticker_parts = self.ticker.split('_', 3)
        for i in range(len(ticker_parts)):
            df[add_cols[i]] = ticker_parts[i]
        if 'transformation' not in df.columns:
            df['transformation'] = None
        return df.loc[:, add_cols + old_cols]


if __name__ == "__main__":

    vins_m = VintageData('USD_INDX_SA', cutoff="2019-06-30", release_lags=[3, 20, 25],  number_firsts=12,
                         shortest=12, sd_ar=5, trend_ar=20, seasonal=10, added_dates=6)
    dfm1 = vins_m.make_grade1()
    dfm1.groupby('release_date').agg(['mean', 'count'])
    dfm2 = vins_m.make_grade2()
    dfmg = vins_m.make_graded(grading=[3, 2.1, 1], upgrades=[12, 24])

    vins_q = VintageData('USD_INDX_SA', release_lags=[3, 20, 25], number_firsts=2, shortest=8, freq='Q',
                         seasonal = 10, added_dates = 4)
    dfq1 = vins_q.make_grade1()
    dfq1.groupby('release_date').agg(['mean', 'count'])
    dfq2 = vins_q.make_grade2()

    vins_w = VintageData('USD_INDX_SA', cutoff="2019-06-30", release_lags=[3, 20, 25],
                         number_firsts=3 * 52, shortest=26, freq='W', seasonal = 10, added_dates = 52)
    dfw1 = vins_w.make_grade1()
    dfw1.groupby('release_date').agg(['mean', 'count'])
    dfw2 = vins_w.make_grade2()

    dfm1.info()


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import statsmodels.api as sm
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.check_availability import reduce_df


def view_grades(df: pd.DataFrame, xcat: str,  cids: List[str] = None,
                start: str = '2000-01-01', end: str = None, grade: str = 'grading',
                title: str = None, ylab: str = None, size: Tuple[float] = (16, 8)):

    """Displays stacked bars of grade composition for category

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcat', 'real_date' and at least one column with values of interest.
    :param <str> xcat: extended categorys to be checked on.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <str> grade: name of column that contains the grades Default is 'grading'.
    :param <str> title: string of chart title; defaults depend on type of range plot.
    :param <str> ylab: y label. Default is no label.
    :param <Tuple[float]> size: Tuple of width and height of graph. Default is (16, 8).

    """
    df, xcats, cids = reduce_df(df, [xcat], cids, start, end, out_all=True)
    df = df[['cid', 'real_date', grade]]

    s_date = df['real_date'].min().strftime('%Y-%m-%d')
    e_date = df['real_date'].max().strftime('%Y-%m-%d')

    if title is None:
        title = f'Daily grade counts for {xcat} from {s_date} to {e_date}'
    if ylab is None:
        ylab = ""

    sns.set_theme(style="darkgrid")
    fig, ax = plt.subplots(figsize=size)

    grades = sorted(df[grade].unique(), reverse=True)
    sns.countplot(x="cid", hue=grade, hue_order=grades, data=df, palette='Blues')

    ax.set_title(title,  fontdict={'fontsize': 16})
    ax.set_xlabel("")
    ax.set_ylabel(ylab)
    ax.xaxis.grid(True)
    handles, labels = ax.get_legend_handles_labels()
    ax.legend(handles=handles[0:], labels=labels[0:])
    plt.show()


if __name__ == "__main__":

    cids = ['NZD', 'AUD', 'CAD', 'GBP']
    xcats = ['XR', 'CRY', 'GROWTH', 'INFL']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD',] = ['2000-01-01', '2020-12-31', 0.1, 1]
    df_cids.loc['CAD',] = ['2001-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP',] = ['2002-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD',] = ['2002-01-01', '2020-09-30', -0.1, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR',] = ['2000-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY',] = ['2000-01-01', '2020-10-30', 1, 2, 0.95, 1]
    df_xcats.loc['GROWTH',] = ['2001-01-01', '2020-10-30', 1, 2, 0.9, 1]
    df_xcats.loc['INFL',] = ['2001-01-01', '2020-10-30', 1, 2, 0.8, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    dfd['grading'] = '3'
    dfd.loc[dfd['real_date'] >= pd.to_datetime('2015-01-01'), 'grading'] = '2.1'
    dfd.loc[dfd['real_date'] >= pd.to_datetime('2016-01-01'), 'grading'] = '1'

    view_grades(dfd, 'CRY', cids=cids, start='2012-01-01')

    dfd.info()



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.check_availability import reduce_df


def view_ranges(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None,
                start: str = '2000-01-01', end: str = None, val: str = 'value',
                kind: str = 'bar', sort_cids_by: str = None,
                title: str = None, ylab: str = None, size: Tuple[float] = (16, 8)):

    """Plot level bars and SD ranges across extended categories and cross sections

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date' and at least one column with values of interest.
    :param <List[str]> xcats: extended categories to be checked on. Default is all in the dataframe.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <str> val: name of column that contains the values of interest. Default is 'value'.
    :param <str> kind: type of range plot to be used. Default is 'bar'; other option is 'box'.
    :param <str> sort_cids_by: criterion for sorting cids on x axis; can be 'mean' and 'std'. Default is original order.
    :param <str> title: string of chart title; defaults depend on type of range plot.
    :param <str> ylab: y label. Default is no label.
    :param <Tuple[float]> size: Tuple of width and height of graph. Default is (16, 8). 

    """

    df, xcats, cids = reduce_df(df, xcats, cids, start, end, out_all=True)

    s_date = df['real_date'].min().strftime('%Y-%m-%d')
    e_date = df['real_date'].max().strftime('%Y-%m-%d')

    sns.set(style="darkgrid")

    if title is None:
        if kind == 'bar':
            title = f'Means and standard deviations from {s_date} to {e_date}'
        elif kind == 'box':
            title = f'Interquartile ranges, extended ranges and outliers from {s_date} to {e_date}'
    if ylab is None:
        ylab = ""

    if sort_cids_by == 'mean':
        dfx = df[df['xcat'] == xcats[0]].groupby(['cid'])[val].mean()
        order = dfx.sort_values(ascending=False).index
    elif sort_cids_by == 'std':
        dfx = df[df['xcat']==xcats[0]].groupby(['cid'])[val].std()
        order = dfx.sort_values(ascending=False).index
    else:
        order = None

    sns.set_theme(style="darkgrid")
    fig, ax = plt.subplots(figsize=size)

    if kind == 'bar':
        ax = sns.barplot(x='cid', y=val, hue='xcat', hue_order=xcats, palette='Paired', data=df, ci='sd', order=order)
    elif kind == 'box':
        ax = sns.boxplot(x='cid', y=val, hue='xcat', hue_order=xcats, palette='Paired', data=df,  order=order)
        ax.xaxis.grid(True)

    ax.set_title(title,  fontdict={'fontsize': 16})
    ax.set_xlabel("")
    ax.set_ylabel(ylab)
    ax.xaxis.grid(True)
    ax.axhline(0, ls='--', linewidth=1, color='black')
    handles, labels = ax.get_legend_handles_labels()
    ax.legend(handles=handles[0:], labels=labels[0:])
    plt.show()


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP']
    xcats = ['XR', 'CRY']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.5, 0.2]
    df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP', ] = ['2012-01-01', '2020-11-30', 0, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]
    df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    view_ranges(dfd, xcats=['XR', 'CRY'], kind='bar', sort_cids_by='mean')
    view_ranges(dfd, xcats=['XR'], cids=cids, kind='box', sort_cids_by='std')
    view_ranges(dfd, xcats=['XR'], cids=cids, kind='box', start='2012-01-01', end='2018-01-01', sort_cids_by='std')





import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import statsmodels.api as sm
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.shape_dfs import categories_df


class CategoryRelations:

    """Class for analyzing and visualizing two categories across a panel

    :param <pd.Dataframe> df: standardized data frame with the following necessary columns:
        'cid', 'xcats', 'real_date' and at least one column with values of interest.
    :param <List[str]> xcats: Exactly two extended categories to be checked on.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the data frame.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <dict> blacklist: cross sections with date ranges that should be excluded from the data frame.
    :param <int> years: Number of years over which data are aggregate. Supersedes freq and does not allow lags,
        Default is None, meaning no multi-year aggregation.
    :param <str> val: name of column that contains the values of interest. Default is 'value'.
    :param <str> freq: letter denoting frequency at which the series are to be sampled.
        This must be one of 'D', 'W', 'M', 'Q', 'A'. Default is 'M'.
    :param <int> lag: Lag (delay of arrival) of second category in periods as set by freq. Default is 0.
    :param <int> fwin: Forward moving average window of first category. Default is 1, i.e no average.
    :param <List[str]> xcat_aggs: Exactly two aggregation methods. Default is 'mean' for both.

    """

    def __init__(self, df: pd.DataFrame, xcats: List[str], cids: List[str] = None, val: str = 'value',
                 start: str = None, end: str = None, blacklist: dict = None, years=None,
                 freq: str = 'M', lag: int = 0, fwin: int = 1, xcat_aggs: List[str] = ('mean', 'mean')):

        """Constructs all attributes for the category relationship to be analyzed"""

        self.xcats = xcats
        self.cids = cids
        self.val = val
        self.freq = freq
        self.lag = lag
        self.years = years
        self.aggs = xcat_aggs

        assert self.freq in ['D', 'W', 'M', 'Q', 'A']
        assert {'cid', 'xcat', 'real_date', val}.issubset(set(df.columns))

        self.df = categories_df(df, xcats, cids, val, start=start, end=end, freq=freq, blacklist=blacklist,
                                years=years, lag=lag, fwin=fwin, xcat_aggs=xcat_aggs)

    def reg_scatter(self, title: str = None, labels: bool = False,
                    size: Tuple[float] = (12, 8), xlab: str = None, ylab: str = None,
                    fit_reg: bool = True, reg_ci: int = 95, reg_order: int = 1, reg_robust: bool = False):

        """Display scatterplot and regression line

        :param <str> title: title of plot. If None (default) an informative title is applied.
        :param <bool> labels: assign a cross-section/period label to each dot. Default is False.
        :param <Tuple[float]> size: width and height of the figure
        :param <str> xlab: x-axis label. Default is no label.
        :param <str> ylab: y-axis label. Default is no label.
        :param <bool> fit_reg: if True (default) adds a regression line.
        :param <int> reg_ci: size of the confidence interval for the regression estimate. Default is 95. Can be None.
        :param <int> reg_order: order of the regression equation. Default is 1 (linear).
        :param <bool> reg_robust: if this will de-weight outliers, which is computationally expensive. Default is False.

        """

        fig, ax = plt.subplots(figsize=size)  # set up figure
        sns.regplot(data=self.df, x=self.xcats[0], y=self.xcats[1],
                    ci=reg_ci, order=reg_order, robust=reg_robust, fit_reg=fit_reg,
                    scatter_kws={'s': 30, 'alpha': 0.5, 'color': 'lightgray'}, line_kws={'lw': 1})

        if labels:
            assert self.freq in ['A', 'Q', 'M'], 'Labels are only possible for monthly or lower frequencies'
            df_labs = self.df.dropna().index.to_frame(index=False)
            if self.years is not None:
                ser_labs = df_labs['cid'] + ' ' + df_labs['real_date']
            elif self.freq == 'A':
                ser_labs = df_labs['cid'] + ' ' + df_labs['real_date'].dt.year.astype(str)
            elif self.freq == 'Q':
                ser_labs = df_labs['cid'] + ' ' + df_labs['real_date'].dt.year.astype(str) + 'Q' \
                           + df_labs['real_date'].dt.quarter.astype(str)
            elif self.freq == 'M':
                ser_labs = df_labs['cid'] + ' ' + df_labs['real_date'].dt.year.astype(str) + '-' \
                           + df_labs['real_date'].dt.month.astype(str)
            for i in range(self.df.shape[0]):
                plt.text(x=self.df[self.xcats[0]][i] + 0, y=self.df[self.xcats[1]][i] + 0, s=ser_labs[i],
                         fontdict=dict(color='black', size=8))

        if title is None and (self.years is None):
            dates = self.df.index.get_level_values('real_date').to_series().dt.strftime('%Y-%m-%d')
            title = f'{self.xcats[0]} and {self.xcats[1]} from {dates.min()} to {dates.max()}'
        elif title is None:
            title = f'{self.xcats[0]} and {self.xcats[1]}'

        ax.set_title(title, fontsize=14)
        if xlab is not None:
            ax.set_xlabel(xlab)
        if ylab is not None:
            ax.set_ylabel(ylab)
        plt.show()

    def ols_table(self):
        """Print statsmodel OLS table of pooled regression"""

        x, y = self.df.dropna().iloc[:, 0], self.df.dropna().iloc[:, 1]
        x_fit = sm.add_constant(x)
        fit_results = sm.OLS(y, x_fit).fit()
        print(fit_results.summary())


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP', 'NZD']
    xcats = ['XR', 'CRY', 'GROWTH', 'INFL']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD',] = ['2000-01-01', '2020-12-31', 0.1, 1]
    df_cids.loc['CAD',] = ['2001-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP',] = ['2002-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD',] = ['2002-01-01', '2020-09-30', -0.1, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR',] = ['2000-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY',] = ['2000-01-01', '2020-10-30', 1, 2, 0.95, 1]
    df_xcats.loc['GROWTH',] = ['2001-01-01', '2020-10-30', 1, 2, 0.9, 1]
    df_xcats.loc['INFL',] = ['2001-01-01', '2020-10-30', 1, 2, 0.8, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    dfc = categories_df(dfd, xcats=['GROWTH', 'CRY'], cids=cids, freq='M', lag=0, xcat_aggs=['mean', 'mean'],
                        start='2000-01-01', years=5)
    cr = CategoryRelations(dfd, xcats=['GROWTH', 'INFL'], cids=cids, freq='M', lag=0, xcat_aggs=['mean', 'mean'],
                           start='1999-01-01', years=10)
    cr.reg_scatter(labels=True)

    black = {'AUD': ['2000-01-01', '2003-12-31'], 'GBP': ['2018-01-01', '2100-01-01']}
    cr = CategoryRelations(dfd, xcats=['GROWTH', 'INFL'], cids=cids, freq='M', xcat_aggs=['mean', 'mean'],
                           start='2000-01-01', years=10, blacklist=black)
    cr.reg_scatter(labels=True)
    cr.ols_table()

    dfc.info()



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from typing import List, Union, Tuple

from macrosynergy.management.simulate_quantamental_data import make_qdf
from macrosynergy.management.check_availability import reduce_df


def correl_matrix(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None,
                   start: str = '2000-01-01', end: str = None, val: str = 'value',
                   title: str = None, size: Tuple[float] = (14, 8), max_color: float=None):

    """Display correlation matrix either across xcats (if more than one xcat_) or cids

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date' and at least one column with values of interest.
    :param <List[str]> xcats: extended categories to be correlated. Default is all in the dataframe.
        If xcats contains only one category the correlation coefficients across sections are displayed.
        If xcats contains more than one category the correlation coefficients across categories are displayed.
    :param <List[str]> cids: cross sections to be correlated. Default is all in the dataframe.
    :param <str> start: earliest date in ISO format. Default is None and earliest date in df is used.
    :param <str> end: latest date in ISO format. Default is None and latest date in df is used.
    :param <str> val: name of column that contains the values of interest. Default is 'value'.
    :param <str> title: chart heading. If none is given, a default title is used.
    :param <Tuple[float]> size: two-element tuple setting width/height of figure. Default is (14, 8).
    :param <float> max_color: maximum values of positive/negative correlation coefficients for color scale
        Default is none. If a value is given it applies symmetrically to positive and negative values.

    """

    xcats = xcats if isinstance(xcats, list) else [xcats]
    min_color = None if max_color is None else -max_color  # define minimum of color scale

    df, xcats, cids = reduce_df(df, xcats, cids, start, end, out_all=True)

    s_date = df['real_date'].min().strftime('%Y-%m-%d')
    e_date = df['real_date'].max().strftime('%Y-%m-%d')

    if len(xcats) == 1:

        df_w = df.pivot(index='real_date', columns='cid', values=val)
        if title is None:
            title = f'Cross-sectional correlation of {xcats[0]} from {s_date} to {e_date}'

    else:

        df_w = df.pivot(index=('cid', 'real_date'), columns='xcat', values=val)
        if title is None:
            title = f'Cross-category correlation from {s_date} to {e_date}'

    sns.set(style="ticks")
    corr = df_w.corr()
    mask = np.triu(np.ones_like(corr, dtype=bool))  # generate mask for upper triangle
    fig, ax = plt.subplots(figsize=size)  # set up figure
    sns.heatmap(corr, mask=mask, cmap='vlag_r', center=0, vmin=min_color, vmax=max_color,
                square=False, linewidths=.5, cbar_kws={"shrink": .5})
    ax.set(xlabel='', ylabel='')
    ax.set_title(title, fontsize=14)

    plt.show()


if __name__ == "__main__":

    cids = ['AUD', 'CAD', 'GBP', 'NZD']
    xcats = ['XR', 'CRY', 'GROWTH', 'INFL']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.1, 1]
    df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP', ] = ['2012-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD', ] = ['2012-01-01', '2020-09-30', -0.1, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.95, 0.5]
    df_xcats.loc['GROWTH', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]
    df_xcats.loc['INFL', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.8, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    correl_matrix(dfd, xcats=xcats, cids=cids, max_color=0.2)
    correl_matrix(dfd, xcats=xcats[0], cids=cids, title='Correlation')

    correl_matrix(dfd, xcats=xcats, cids=cids)




import numpy as np
import pandas as pd
from typing import List, Union, Tuple
from statsmodels.tsa.arima_process import ArmaProcess
import matplotlib.pyplot as plt
import datetime


def simulate_ar(nobs: int, mean: float = 0, sd_mult: float = 1, ar_coef: float = 0.75):
    """Create an autocorrelated dataseries as numpy array

    :param <int> nobs: number of observations.
    :param <float> mean: mean of values, default is zero.
    :param <float> sd_mult: standard deviation multipliers of values, default is 1. This affects non-zero means.
    :param <float> ar_coef: autoregression coefficient (between 0 and 1): default is 0.75.

    return <np.array>: autocorrelated data series.
    """

    ar_params = np.r_[1, -ar_coef]  # define relative parameters for creating an AR process
    ar_proc = ArmaProcess(ar_params)  # define ARMA process
    ser = ar_proc.generate_sample(nobs)
    ser = ser + mean - np.mean(ser)
    return sd_mult * ser / np.std(ser)


def make_qdf(df_cids: pd.DataFrame, df_xcats: pd.DataFrame, back_ar: float = 0):
    """
    Make quantamental dataframe with basic columns: 'cid', 'xcat', 'real_date', 'value'

    :param <pd.DataFrame> df_cids: dataframe with parameters by cid. Row indices are cross-sections. Columns are:
        'earliest': string of earliest date (ISO) for which country values are available;
        'latest': string of latest date (ISO) for which country values are available;
        'mean_add': float of country-specific addition to any category's mean;
        'sd_mult': float of country-specific multiplier of an category's standard deviation.
    :param <pd.DataFrame> df_xcats: dataframe with parameters by xcat. Row indices are cross-sections. Columns are:
        'earliest': string of earliest date (ISO) for which category values are available;
        'latest': string of latest date (ISO) for which category values are available;
        'mean_add': float of category-specific addition;
        'sd_mult': float of country-specific multiplier of an category's standard deviation;
        'ar_coef': float between 0 and 1 denoting set autocorrelation of the category;
        'back_coef': float, coefficient with which communal (mean 0, SD 1) background factor is added to categoy values.
    :param <float> back_ar: float between 0 and 1 denoting set autocorrelation of the background factor.
        Default is zero.

    :return <pd.DataFrame>: basic quantamental dataframe according to specifications.

    """
    qdf_cols = ['cid', 'xcat', 'real_date', 'value']
    df_out = pd.DataFrame(columns=qdf_cols)

    if any(df_xcats['back_coef'] != 0):

        sdate = min(min(df_cids.loc[:, 'earliest']), min(df_xcats.loc[:, 'earliest']))
        edate = max(max(df_cids.loc[:, 'latest']), max(df_xcats.loc[:, 'latest']))
        all_days = pd.date_range(sdate, edate)
        work_days = all_days[all_days.weekday < 5]
        ser = simulate_ar(len(work_days), mean=0, sd_mult=1, ar_coef=back_ar)
        df_back = pd.DataFrame(index=work_days, columns=['value'])
        df_back['value'] = ser

    for cid in df_cids.index:
        for xcat in df_xcats.index:

            df_add = pd.DataFrame(columns=qdf_cols)

            sdate = pd.to_datetime(max(df_cids.loc[cid, 'earliest'], df_xcats.loc[xcat, 'earliest']))
            edate = pd.to_datetime(min(df_cids.loc[cid, 'latest'], df_xcats.loc[xcat, 'latest']))
            all_days = pd.date_range(sdate, edate)
            work_days = all_days[all_days.weekday < 5]

            df_add['real_date'] = work_days
            df_add['cid'] = cid
            df_add['xcat'] = xcat

            ser_mean = df_cids.loc[cid, 'mean_add'] + df_xcats.loc[xcat, 'mean_add']
            ser_sd = df_cids.loc[cid, 'sd_mult'] * df_xcats.loc[xcat, 'sd_mult']
            ser_arc = df_xcats.loc[xcat, 'ar_coef']
            df_add['value'] = simulate_ar(len(work_days), mean=ser_mean, sd_mult=ser_sd, ar_coef=ser_arc)

            back_coef = df_xcats.loc[xcat, 'back_coef']
            if back_coef != 0:  # add influence of communal background series
                df_add['value'] = df_add['value'] + \
                                  back_coef * df_back.loc[df_add['real_date'], 'value'].reset_index(drop=True)

            df_out = df_out.append(df_add)

    return df_out.reset_index(drop=True)


if __name__ == "__main__":

    ser_ar = simulate_ar(100, mean=0, sd_mult=1, ar_coef=0.75)

    cids = ['AUD', 'CAD', 'GBP']
    xcats = ['XR', 'CRY']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.5, 2]
    df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP', ] = ['2011-01-01', '2020-11-30', -0.2, 0.5]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]
    df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]

    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)









import numpy as np
import pandas as pd
from typing import List, Union, Tuple
import random

from macrosynergy.management.simulate_quantamental_data import make_qdf


def reduce_df(df: pd.DataFrame, xcats: List[str] = None,  cids: List[str] = None,
              start: str = None, end: str = None, blacklist: dict = None, out_all: bool = False):
    """
    Filter dataframe by xcats and cids and notify about missing xcats and cids

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date'.
    :param <List[str]> xcats: extended categories to be checked on. Default is all in the dataframe.
    :param <List[str]> cids: cross sections to be checked on. Default is all in the dataframe.
    :param <str> start: string representing earliest date. Default is None.
    :param <str> end: string representing the latest date. Default is None.
    :param <dict> blacklist: cross sections with date ranges that should be excluded from the data frame.
        If one cross section has several blacklist periods append numbers to the cross section code.
    :param <bool> out_all: if True the function returns reduced dataframe and selected/available xcats and cids.
        Default is False, i.e. only the dataframe is returned

    :return <pd.Dataframe>: reduced dataframe
        or (for out_all True) dataframe and avalaible and selected xcats and cids
    """

    dfx = df[df['real_date'] >= pd.to_datetime(start)] if start is not None else df
    dfx = dfx[dfx['real_date'] <= pd.to_datetime(end)] if end is not None else dfx

    if blacklist is not None:
        for key, value in blacklist.items():
            filt1 = dfx['cid'] == key[:3]
            filt2 = dfx['real_date'] >= pd.to_datetime(value[0])
            filt3 = dfx['real_date'] <= pd.to_datetime(value[1])
            dfx = dfx[~(filt1 & filt2 & filt3)]

    xcats_in_df = dfx['xcat'].unique()
    if xcats is None:
        xcats = sorted(xcats_in_df)
    else:
        missing = sorted(set(xcats) - set(xcats_in_df))
        if len(missing) > 0:
            print(f'Missing cross sections: {missing}')
            xcats.remove(missing)

    dfx = dfx[dfx['xcat'].isin(xcats)]

    cids_in_df = dfx['cid'].unique()
    if cids is None:
        cids = sorted(cids_in_df)
    else:
        if not isinstance(cids, list):
           cids = [cids]
        missing = sorted(set(cids) - set(cids_in_df))
        if len(missing) > 0:
            print(f'Missing cross sections: {missing}')
        cids = sorted(list(set(cids).intersection(set(cids_in_df))))
        dfx = dfx[dfx['cid'].isin(cids)]

    if out_all:
        return dfx, xcats, cids
    else:
        return dfx


def categories_df(df: pd.DataFrame, xcats: List[str], cids: List[str] = None, val: str = 'value',
                  start: str = None, end: str = None, blacklist: dict = None, years: int = None,
                  freq: str = 'M', lag: int = 0, fwin: int = 1, xcat_aggs: List[str] = ('mean', 'mean')):

    """Create custom two-categories dataframe with appropriate frequency and lags suitable for analysis

    :param <pd.Dataframe> df: standardized dataframe with the following necessary columns:
        'cid', 'xcats', 'real_date' and at least one column with values of interest.
    :param <List[str]> xcats: exactly two extended categories whose relationship is to be analyzed.
    :param <List[str]> cids: cross sections to be included. Default is all in the dataframe.
    :param <str> start: earliest date in ISO format. Default is None, i.e. earliest date in data frame is used.
    :param <str> end: latest date in ISO format. Default is None, i.e. latest date in data frame is used.
    :param <dict> blacklist: cross sections with date ranges that should be excluded from the data frame.
        If one cross section has several blacklist periods append numbers to the cross section code.
    :param <int> years: Number of years over which data are aggregated. Supersedes freq and does not allow lags,
        Default is None, i.e. no multi-year aggregation.
    :param <str> val: name of column that contains the values of interest. Default is 'value'.
    :param <str> freq: letter denoting frequency at which the series are to be sampled.
        This must be one of 'D', 'W', 'M', 'Q', 'A'. Default is 'M'.
    :param <int> lag: Lag (delay of arrival) of second category in periods as set by freq. Default is 0.
        Note: for analyses with dependent and explanatory categories, the second takes the role of the explanatory.
    :param <int> fwin: Forward moving average window of first category. Default is 1, i.e no average.
        Note: this parameter is used mainly for target returns as dependent variables.
    :param <List[str]> xcat_aggs: Exactly two aggregation methods. Default is 'mean' for both.

    :return custom data frame with columns for the two categories indexed by cross sections and periods.

    """

    assert freq in ['D', 'W', 'M', 'Q', 'A']
    assert not (years is not None) & (lag != 0), 'Lags cannot be applied to year groups'

    df, xcats, cids = reduce_df(df, xcats, cids, start, end, blacklist, out_all=True)

    col_names = ['cid', 'xcat', 'real_date', val]
    dfc = pd.DataFrame(columns=col_names)

    if years is None:
        for i in range(2):
            dfw = df[df['xcat'] == xcats[i]].pivot(index='real_date', columns='cid', values=val)
            dfw = dfw.resample(freq).agg(xcat_aggs[i])  # frequency conversion
            if (i == 0) & (fwin > 1):
                dfw = dfw.rolling(window=fwin).mean().shift(1-fwin)
            if (i == 1) & (lag > 0):
                dfw = dfw.shift(lag)  # lag second category for late arrival
            dfx = pd.melt(dfw.reset_index(), id_vars=['real_date'], value_vars=cids, value_name=val)
            dfx['xcat'] = xcats[i]
            dfc = dfc.append(dfx[col_names])
    else:
        s_year = pd.to_datetime(start).year
        e_year = df['real_date'].max().year + 1

        s_years = range(s_year, e_year, years)
        year_groups = {}
        for y in s_years:
            ey = y + years - 1 if (y + years - 1) <= e_year else 'now'
            y_key = f'{y} - {ey}'
            y_value = [i for i in range(y, y + years)]
            year_groups[y_key] = y_value

        def translate(year):
            return np.array(list(year_groups.keys()))[[year in l for l in list(year_groups.values())]][0]

        df['custom_date'] = df['real_date'].dt.year.apply(translate)
        for i in range(2):
            dfx = df[df['xcat'] == xcats[i]]
            dfx = dfx.groupby(['xcat', 'cid', 'custom_date']).agg(xcat_aggs[i]).reset_index()
            dfx = dfx.rename(columns={"custom_date": "real_date"})
            dfc = dfc.append(dfx[col_names])

    return dfc.pivot(index=('cid', 'real_date'), columns='xcat', values=val).dropna()[xcats]


if __name__ == "__main__":

    cids = ['NZD', 'AUD', 'GBP', 'CAD']
    xcats = ['XR', 'CRY', 'GROWTH', 'INFL']
    df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
    df_cids.loc['AUD',] = ['2000-01-01', '2020-12-31', 0.1, 1]
    df_cids.loc['CAD',] = ['2001-01-01', '2020-11-30', 0, 1]
    df_cids.loc['GBP',] = ['2002-01-01', '2020-11-30', 0, 2]
    df_cids.loc['NZD',] = ['2002-01-01', '2020-09-30', -0.1, 2]

    df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
    df_xcats.loc['XR',] = ['2000-01-01', '2020-12-31', 0.1, 1, 0, 0.3]
    df_xcats.loc['CRY',] = ['2000-01-01', '2020-10-30', 1, 2, 0.95, 1]
    df_xcats.loc['GROWTH',] = ['2001-01-01', '2020-10-30', 1, 2, 0.9, 1]
    df_xcats.loc['INFL',] = ['2001-01-01', '2020-10-30', 1, 2, 0.8, 0.5]

    random.seed(2)
    dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)

    dfd_x1 = reduce_df(dfd, xcats=xcats, cids=cids[0], start='2012-01-01', end='2018-01-31')
    dfd_x = reduce_df(dfd, xcats=xcats, cids=cids, start='2012-01-01', end='2018-01-31')

    black = {'AUD': ['2000-01-01', '2003-12-31'], 'GBP': ['2018-01-01', '2100-01-01']}

    dfd_xb = reduce_df(dfd, xcats=xcats, cids=cids, blacklist=black)

    dfc1 = categories_df(dfd, xcats=['GROWTH', 'CRY'], cids=cids, freq='M', lag=0, xcat_aggs=['mean', 'mean'],
                         start='2000-01-01', blacklist=black)
    dfc2 = categories_df(dfd, xcats=['GROWTH', 'CRY'], cids=cids, freq='M', lag=0, fwin=3, xcat_aggs=['mean', 'mean'],
                         start='2000-01-01', blacklist=black)

    black = {'AUD_1': ['2000-01-01', '2009-12-31'], 'AUD_2': ['2018-01-01', '2100-01-01']}
    dfc3 = categories_df(dfd, xcats=['GROWTH', 'CRY'], cids=cids, freq='M', lag=0, xcat_aggs=['mean', 'mean'],
                         start='2000-01-01', blacklist=black, years=10)

    dfd_xb.tail()


import subprocess
import sys
import unittest


class TestPackage(unittest.TestCase):
    def test_lazy_imports(self):
        # Check that when qstools imported correctly
        cmd = ("import macrosynergy as msy; "
               "import sys; ")

        cmd = sys.executable + ' -c "' + cmd + '"'
        p = subprocess.Popen(cmd, shell=True, close_fds=True)
        p.wait()
        rc = p.returncode
        assert rc == 0

    def test_docstring_optimization_compat(self):
        # Check that importing with stripped docstrings does not raise
        cmd = sys.executable + ' -OO -c "import macrosynergy as qs"'
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        out = p.communicate()
        rc = p.returncode
        assert rc == 0, out


if __name__ == '__main__':
    unittest.main()



from .shape_dfs import reduce_df, categories_df
from .check_availability import check_availability, visual_paneldates, check_enddates, check_startyears\
    , missing_in_df
from .simulate_vintage_data import VintageData
from .simulate_quantamental_data import make_qdf

__all__ = ['check_availability', 'visual_paneldates', 'check_enddates', 'check_startyears',
           'reduce_df', 'missing_in_df', 'VintageData', 'make_qdf']


import unittest
import random
import numpy as np
import pandas as pd
from macrosynergy.management.simulate_quantamental_data import make_qdf, simulate_ar

cids = ['AUD', 'CAD', 'GBP']
xcats = ['XR', 'CRY']
df_cids = pd.DataFrame(index=cids, columns=['earliest', 'latest', 'mean_add', 'sd_mult'])
df_cids.loc['AUD', ] = ['2010-01-01', '2020-12-31', 0.5, 2]
df_cids.loc['CAD', ] = ['2011-01-01', '2020-11-30', 0, 1]
df_cids.loc['GBP', ] = ['2011-01-01', '2020-11-30', -0.2, 0.5]

df_xcats = pd.DataFrame(index=xcats, columns=['earliest', 'latest', 'mean_add', 'sd_mult', 'ar_coef', 'back_coef'])
df_xcats.loc['XR', ] = ['2010-01-01', '2020-12-31', 0, 1, 0, 0.3]
df_xcats.loc['CRY', ] = ['2011-01-01', '2020-10-30', 1, 2, 0.9, 0.5]

random.seed(1)
dfd = make_qdf(df_cids, df_xcats, back_ar=0.75)


def ar1_coef(x):
    return np.corrcoef(np.array([x[:-1], x[1:]]))[0, 1]


def cor_coef(df, ticker_x, ticker_y):
    x = ticker_x.split('_', 1)
    y = ticker_y.split('_', 1)
    filt_x = (df['cid'] == x[0]) & (dfd['xcat'] == x[1])
    filt_y = (df['cid'] == y[0]) & (dfd['xcat'] == y[1])
    dfd_x = dfd.loc[filt_x,].set_index('real_date')['value']
    dfd_y = dfd.loc[filt_y,].set_index('real_date')['value']
    dfd_xy = pd.merge(dfd_x, dfd_y, how='inner', left_index=True, right_index=True)
    return dfd_xy.corr().iloc[0, 1]


class Test_All(unittest.TestCase):

    def test_simulate_ar(self):

        random.seed(1)
        ser_ar = simulate_ar(100, mean=2, sd_mult=3, ar_coef=0.75)
        self.assertGreater(ar1_coef(ser_ar), 0.25)
        self.assertEqual(np.round(np.std(ser_ar), 2), 3)
        self.assertGreater(np.mean(ser_ar), 2)

    def test_qdf_starts(self):

        filt1 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'CRY')
        self.assertEqual(np.min(dfd.loc[filt1, 'real_date']), pd.to_datetime('2011-01-03'))
        filt1 = (dfd['cid'] == 'GBP') & (dfd['xcat'] == 'XR')
        self.assertEqual(np.min(dfd.loc[filt1, 'real_date']), pd.to_datetime('2011-01-03'))

    def test_qdf_ends(self):

        filt1 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'CRY')
        self.assertEqual(np.max(dfd.loc[filt1, 'real_date']), pd.to_datetime('2020-10-30'))
        filt1 = (dfd['cid'] == 'GBP') & (dfd['xcat'] == 'XR')
        self.assertEqual(np.max(dfd.loc[filt1, 'real_date']), pd.to_datetime('2020-11-30'))

    def test_qdf_correl(self):

        self.assertGreater(cor_coef(dfd, 'AUD_XR', 'CAD_XR'), 0)
        self.assertGreater(cor_coef(dfd, 'AUD_XR', 'GBP_XR'), 0)

    def test_qdf_ar(self):

        filt1 = (dfd['cid'] == 'AUD') & (dfd['xcat'] == 'CRY')
        self.assertGreater(ar1_coef(dfd.loc[filt1, 'value']), 0.25)
        filt1 = (dfd['cid'] == 'CAD') & (dfd['xcat'] == 'CRY')
        self.assertGreater(ar1_coef(dfd.loc[filt1, 'value']), 0.25)
        filt1 = (dfd['cid'] == 'GBP') & (dfd['xcat'] == 'CRY')
        self.assertGreater(ar1_coef(dfd.loc[filt1, 'value']), 0.25)


if __name__ == '__main__':

    unittest.main()
